@@ -1,21 +1,42 @@
 import inspect
 import types
-from typing import *
-from typing_extensions import Self
+import typing as t
+import typing_extensions as te
 
 from . import errors
 
 __all__ = ["CallFrame"]
 
 
+# `inspect.currentframe()` is *much* faster than `inspect.stack()`, but may not be available.
+# Choose the appropriate implementation.
+if inspect.currentframe() is None:
+
+    def get_frame(i: int) -> inspect.FrameInfo:  # type: ignore (override)
+        return inspect.stack()[i]
+
+else:
+
+    def get_frame(i: int) -> types.FrameType:  # type: ignore (override)
+        frame = inspect.currentframe()
+
+        try:
+            for _ in range(i):
+                frame = frame.f_back  # type: ignore
+        except AttributeError:
+            raise IndexError
+
+        if frame is None:
+            raise IndexError
+
+        return frame
+
+
 class CallFrame:
     """
     Represents a call frame - an element of the call stack.
     It keeps track of local and closure variables.
 
-    Although ``CallFrame`` does not inherit from :data:`types.FrameType`,
-    they can be used just like regular frame objects.
-
     Note that storing CallFrames in variables can create reference
     cycles where a frame contains a reference to itself. To avoid
     this, CallFrames can be used as context managers - upon exit,
@@ -28,40 +49,61 @@ class CallFrame:
 
     __slots__ = ("_frame",)
 
-    def __init__(self, frame: types.FrameType):
+    def __init__(self, frame: t.Union[types.FrameType, inspect.FrameInfo]):
         """
-        Creates a new ``CallFrame`` from a :data:`types.FrameType` object.
+        Creates a new ``CallFrame`` from a :data:`types.FrameType` or :cls:`inspect.FrameInfo`
+        object.
 
         :param frame: An existing frame object
         """
+        if isinstance(frame, inspect.FrameInfo):
+            frame = frame.frame
+
         self._frame = frame
 
     @classmethod
-    def current(cls) -> Self:
+    def current(cls) -> te.Self:
         """
         Retrieves the current call frame.
         """
-        return cls(inspect.currentframe().f_back)  # type: ignore
+        return cls.up(1)  # up(1) because we need to skip the implementation of `current`
 
-    def __getattr__(self, attr: str):
-        return getattr(self._frame, attr)
+    @classmethod
+    def up(cls, n: int, /) -> te.Self:
+        """
+        Retrieves the `n`th frame (from the bottom) from the call stack. In
+        other words, `up(0)` is equivalent to `CallFrame.current()`.
+
+        :raises IndexError: If `n` is larger than the call stack
+        """
+        return cls(get_frame(n + 2))
+
+    @classmethod
+    def iter(cls) -> t.Iterator[te.Self]:
+        call_frame = cls.up(1)
+
+        while call_frame is not None:
+            yield call_frame
+            call_frame = call_frame.parent
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, __class__):
             return self._frame == other._frame
         elif isinstance(other, types.FrameType):
             return self._frame == other
+        elif isinstance(other, inspect.FrameInfo):
+            return self._frame == other.frame
         else:
             return NotImplemented
 
-    def __enter__(self) -> Self:
+    def __enter__(self) -> te.Self:
         return self
 
     def __exit__(self, exc_type: object, exc_val: object, exc_tb: object) -> None:
         del self._frame
 
     @property
-    def parent(self) -> Optional[Self]:
+    def parent(self) -> t.Optional[te.Self]:
         """
         Returns the next frame one level higher on the call stack.
         """
@@ -73,26 +115,33 @@ class CallFrame:
         return cls(parent)
 
     @property
-    def builtins(self) -> Dict[str, Any]:
+    def builtins(self) -> t.Dict[str, t.Any]:
         """
         Returns the builtins seen by this frame
         """
         return self._frame.f_builtins
 
     @property
-    def globals(self) -> Dict[str, Any]:
+    def globals(self) -> t.Dict[str, t.Any]:
         """
         Returns the global scope seen by this frame
         """
         return self._frame.f_globals
 
     @property
-    def locals(self) -> Dict[str, Any]:
+    def locals(self) -> t.Dict[str, t.Any]:
         """
         Returns the frame's local variable scope
         """
         return self._frame.f_locals
 
+    @property
+    def current_line_number(self) -> int:
+        """
+        Returns the name of the file in which this frame's code was defined
+        """
+        return self._frame.f_lineno
+
     @property
     def code_object(self) -> types.CodeType:
         """
@@ -164,7 +213,7 @@ class CallFrame:
         :return: The calling function object
         """
 
-        def iter_candidate_functions() -> Iterator[types.FunctionType]:
+        def iter_candidate_functions() -> t.Iterator[types.FunctionType]:
             funcname = self.scope_name
 
             try:
