@@ -1,8 +1,9 @@
 import inspect
-from typing import *  # type: ignore
-from typing_extensions import Self
+import typing as t
+import typing_extensions as te
 
 from ._utils import PARAM_EMPTY
+from .types import ForwardRefContext, TypeAnnotation, NONE
 
 __all__ = ["Parameter"]
 
@@ -20,7 +21,7 @@ class Parameter(inspect.Parameter):
     This class adds a new special value for the ``default`` attribute: :attr:`Parameter.missing`.
     """
 
-    __slots__ = ()
+    __slots__ = ("forward_ref_context",)
 
     #: A special class-level marker that can be used to specify
     #: that the parameter is optional, but doesn't have a (known)
@@ -44,8 +45,9 @@ class Parameter(inspect.Parameter):
         self,
         name: str,
         kind: inspect._ParameterKind = inspect.Parameter.POSITIONAL_OR_KEYWORD,
-        default: Any = PARAM_EMPTY,
-        annotation: Any = PARAM_EMPTY,
+        default: t.Any = PARAM_EMPTY,
+        annotation: t.Any = PARAM_EMPTY,
+        forward_ref_context: t.Optional[ForwardRefContext] = None,
     ):
         """
         :param name: The parameter's name
@@ -61,19 +63,34 @@ class Parameter(inspect.Parameter):
 
         super().__init__(name, kind, default=default, annotation=annotation)
 
+        self.forward_ref_context = forward_ref_context
+
     @classmethod
-    def from_parameter(cls, parameter: inspect.Parameter) -> Self:
+    def from_parameter(cls, parameter: inspect.Parameter) -> te.Self:
         """
         Creates a new :class:`Parameter` instance from an :class:`inspect.Parameter` instance.
 
         :param parameter: An :class:`inspect.Parameter` instance
         :return: A new :class:`Parameter` instance
         """
+        if isinstance(parameter, cls):
+            return parameter
+        else:
+            return cls._from_inspect_parameter(parameter)
+
+    @classmethod
+    def _from_inspect_parameter(
+        cls,
+        parameter: inspect.Parameter,
+        *,
+        forward_ref_context: t.Optional[ForwardRefContext] = None,
+    ) -> te.Self:
         return cls(
             parameter.name,
             kind=parameter.kind,
             default=parameter.default,
             annotation=parameter.annotation,
+            forward_ref_context=forward_ref_context,
         )
 
     @property
@@ -112,6 +129,38 @@ class Parameter(inspect.Parameter):
         """
         return self.default is not Parameter.empty
 
+    def replace(  # type: ignore (invalid override)
+        self,
+        *,
+        name: str = NONE,  # type: ignore
+        kind: inspect._ParameterKind = NONE,  # type: ignore
+        default: object = NONE,
+        annotation: TypeAnnotation = NONE,
+        forward_ref_context: t.Optional[ForwardRefContext] = NONE,  # type: ignore
+    ) -> te.Self:
+        if name is NONE:
+            name = self.name
+
+        if kind is NONE:
+            kind = self.kind
+
+        if default is NONE:
+            default = self.default
+
+        if annotation is NONE:
+            annotation = self.annotation
+
+        if forward_ref_context is NONE:
+            forward_ref_context = self.forward_ref_context
+
+        return type(self)(
+            name=name,
+            kind=kind,
+            default=default,
+            annotation=annotation,
+            forward_ref_context=forward_ref_context,
+        )
+
     def _to_string(
         self,
         implicit_typing: bool,
