@@ -1,7 +1,9 @@
+from .forward_ref import *
 from .type_info import *
 from .introspection import *
 from .instance_check import *
 from .subtype_check import *
 from .type_compat import *
 from .misc import *
+from .misc2 import *
 from .i_hate_circular_imports import *
