@@ -9,6 +9,7 @@ import typing
 import typing_extensions
 from typing import *
 
+from . import _compat
 from .i_hate_circular_imports import parameterize
 from ..classes import safe_is_subclass
 from ..types import Type_, GenericAliases, TypeParameter
@@ -61,22 +62,19 @@ def _resolve_dotted_name(name: str) -> object:
 
 
 @overload
-def _resolve_dotted_names(names: Dict[str, T]) -> Dict[object, T]:
-    ...
+def _resolve_dotted_names(names: Dict[str, T]) -> Dict[object, T]: ...
 
 
 @overload
-def _resolve_dotted_names(names: Tuple[str, ...]) -> Tuple[object, ...]:
-    ...
+def _resolve_dotted_names(names: Tuple[str, ...]) -> Tuple[object, ...]: ...
 
 
 @overload
-def _resolve_dotted_names(names: Iterable[str]) -> Iterable[object]:
-    ...
+def _resolve_dotted_names(names: Iterable[str]) -> Iterable[object]: ...
 
 
 def _resolve_dotted_names(
-    names: Union[Dict[str, T], Iterable[str]]
+    names: Union[Dict[str, T], Iterable[str]],
 ) -> Union[Dict[object, T], Iterable[object]]:
     """
     ::
@@ -112,7 +110,7 @@ def _resolve_dotted_names(
 
             result.append(value)
 
-        return type(names)(result)
+        return type(names)(result)  # type: ignore
 
 
 # NOTE: The following function implementations work in python 3.5.
@@ -146,7 +144,7 @@ def _is_variadic_generic(type_: Type_):
     return _is_in(type_, VARIADIC_GENERICS)
 
 
-GENERIC_INHERITANCE = {
+_GENERIC_INHERITANCE = {
     "typing.Type": [("Generic", TypeVar("CT_co", covariant=True))],
     "typing.Annotated": [("Generic", T_co)],
     "typing.ClassVar": [("Generic", T_co)],
@@ -234,75 +232,79 @@ GENERIC_INHERITANCE = {
     "typing.MutableSequence": [("Sequence", T)],
 }
 if sys.version_info >= (3, 9):
-    GENERIC_INHERITANCE.update(
+    _GENERIC_INHERITANCE.update(
         {
-            "builtins.tuple": GENERIC_INHERITANCE["typing.Tuple"],
-            "builtins.list": GENERIC_INHERITANCE["typing.List"],
-            "builtins.dict": GENERIC_INHERITANCE["typing.Dict"],
-            "builtins.set": GENERIC_INHERITANCE["typing.Set"],
-            "builtins.frozenset": GENERIC_INHERITANCE["typing.FrozenSet"],
-            "builtins.type": GENERIC_INHERITANCE["typing.Type"],
-            "collections.deque": GENERIC_INHERITANCE["typing.Deque"],
-            "collections.defaultdict": GENERIC_INHERITANCE["typing.DefaultDict"],
-            "collections.OrderedDict": GENERIC_INHERITANCE["typing.OrderedDict"],
-            "collections.Counter": GENERIC_INHERITANCE["typing.Counter"],
-            "collections.ChainMap": GENERIC_INHERITANCE["typing.ChainMap"],
-            "collections.abc.Awaitable": GENERIC_INHERITANCE["typing.Awaitable"],
-            "collections.abc.Coroutine": GENERIC_INHERITANCE["typing.Coroutine"],
-            "collections.abc.AsyncIterable": GENERIC_INHERITANCE["typing.AsyncIterable"],
-            "collections.abc.AsyncIterator": GENERIC_INHERITANCE["typing.AsyncIterator"],
-            "collections.abc.AsyncGenerator": GENERIC_INHERITANCE["typing.AsyncGenerator"],
-            "collections.abc.Iterable": GENERIC_INHERITANCE["typing.Iterable"],
-            "collections.abc.Iterator": GENERIC_INHERITANCE["typing.Iterator"],
-            "collections.abc.Generator": GENERIC_INHERITANCE["typing.Generator"],
-            "collections.abc.Reversible": GENERIC_INHERITANCE["typing.Reversible"],
-            "collections.abc.Container": GENERIC_INHERITANCE["typing.Container"],
-            "collections.abc.Collection": GENERIC_INHERITANCE["typing.Collection"],
-            "collections.abc.Callable": GENERIC_INHERITANCE["typing.Callable"],
-            "collections.abc.Set": GENERIC_INHERITANCE["typing.AbstractSet"],
-            "collections.abc.MutableSet": GENERIC_INHERITANCE["typing.MutableSet"],
-            "collections.abc.Mapping": GENERIC_INHERITANCE["typing.Mapping"],
-            "collections.abc.MutableMapping": GENERIC_INHERITANCE["typing.MutableMapping"],
-            "collections.abc.Sequence": GENERIC_INHERITANCE["typing.Sequence"],
-            "collections.abc.MutableSequence": GENERIC_INHERITANCE["typing.MutableSequence"],
-            "collections.abc.ByteString": GENERIC_INHERITANCE["typing.ByteString"],
-            "collections.abc.MappingView": GENERIC_INHERITANCE["typing.MappingView"],
-            "collections.abc.KeysView": GENERIC_INHERITANCE["typing.KeysView"],
-            "collections.abc.ItemsView": GENERIC_INHERITANCE["typing.ItemsView"],
-            "collections.abc.ValuesView": GENERIC_INHERITANCE["typing.ValuesView"],
-            "contextlib.AbstractContextManager": GENERIC_INHERITANCE["typing.ContextManager"],
-            "contextlib.AbstractAsyncContextManager": GENERIC_INHERITANCE[
+            "builtins.tuple": _GENERIC_INHERITANCE["typing.Tuple"],
+            "builtins.list": _GENERIC_INHERITANCE["typing.List"],
+            "builtins.dict": _GENERIC_INHERITANCE["typing.Dict"],
+            "builtins.set": _GENERIC_INHERITANCE["typing.Set"],
+            "builtins.frozenset": _GENERIC_INHERITANCE["typing.FrozenSet"],
+            "builtins.type": _GENERIC_INHERITANCE["typing.Type"],
+            "collections.deque": _GENERIC_INHERITANCE["typing.Deque"],
+            "collections.defaultdict": _GENERIC_INHERITANCE["typing.DefaultDict"],
+            "collections.OrderedDict": _GENERIC_INHERITANCE["typing.OrderedDict"],
+            "collections.Counter": _GENERIC_INHERITANCE["typing.Counter"],
+            "collections.ChainMap": _GENERIC_INHERITANCE["typing.ChainMap"],
+            "collections.abc.Awaitable": _GENERIC_INHERITANCE["typing.Awaitable"],
+            "collections.abc.Coroutine": _GENERIC_INHERITANCE["typing.Coroutine"],
+            "collections.abc.AsyncIterable": _GENERIC_INHERITANCE["typing.AsyncIterable"],
+            "collections.abc.AsyncIterator": _GENERIC_INHERITANCE["typing.AsyncIterator"],
+            "collections.abc.AsyncGenerator": _GENERIC_INHERITANCE["typing.AsyncGenerator"],
+            "collections.abc.Iterable": _GENERIC_INHERITANCE["typing.Iterable"],
+            "collections.abc.Iterator": _GENERIC_INHERITANCE["typing.Iterator"],
+            "collections.abc.Generator": _GENERIC_INHERITANCE["typing.Generator"],
+            "collections.abc.Reversible": _GENERIC_INHERITANCE["typing.Reversible"],
+            "collections.abc.Container": _GENERIC_INHERITANCE["typing.Container"],
+            "collections.abc.Collection": _GENERIC_INHERITANCE["typing.Collection"],
+            "collections.abc.Callable": _GENERIC_INHERITANCE["typing.Callable"],
+            "collections.abc.Set": _GENERIC_INHERITANCE["typing.AbstractSet"],
+            "collections.abc.MutableSet": _GENERIC_INHERITANCE["typing.MutableSet"],
+            "collections.abc.Mapping": _GENERIC_INHERITANCE["typing.Mapping"],
+            "collections.abc.MutableMapping": _GENERIC_INHERITANCE["typing.MutableMapping"],
+            "collections.abc.Sequence": _GENERIC_INHERITANCE["typing.Sequence"],
+            "collections.abc.MutableSequence": _GENERIC_INHERITANCE["typing.MutableSequence"],
+            "collections.abc.ByteString": _GENERIC_INHERITANCE["typing.ByteString"],
+            "collections.abc.MappingView": _GENERIC_INHERITANCE["typing.MappingView"],
+            "collections.abc.KeysView": _GENERIC_INHERITANCE["typing.KeysView"],
+            "collections.abc.ItemsView": _GENERIC_INHERITANCE["typing.ItemsView"],
+            "collections.abc.ValuesView": _GENERIC_INHERITANCE["typing.ValuesView"],
+            "contextlib.AbstractContextManager": _GENERIC_INHERITANCE["typing.ContextManager"],
+            "contextlib.AbstractAsyncContextManager": _GENERIC_INHERITANCE[
                 "typing.AsyncContextManager"
             ],
             "re.Match": [("Generic", AnyStr)],
             "re.Pattern": [("Generic", AnyStr)],
         }
     )
-GENERIC_INHERITANCE = _resolve_dotted_names(GENERIC_INHERITANCE)
+GENERIC_INHERITANCE = _resolve_dotted_names(_GENERIC_INHERITANCE)
 
 
-PARAMETERIZED_GENERIC_META = (
-    "types.GenericAlias",  # py3.9+
-    "typing._GenericAlias",  # py3.8+
-    "typing.GenericMeta",  # py3.5-3.7
+PARAMETERIZED_GENERIC_META = _resolve_dotted_names(
+    (
+        "types.GenericAlias",  # py3.9+
+        "typing._GenericAlias",  # py3.8+
+        "typing.GenericMeta",  # py3.5-3.7
+    )
 )
-PARAMETERIZED_GENERIC_META = _resolve_dotted_names(PARAMETERIZED_GENERIC_META)
 
 
 def _get_type_parameters(type_):
     if sys.version_info >= (3, 10):
         if isinstance(type_, types.UnionType):
-            return type_.__parameters__  # type: ignore
+            return type_.__parameters__
 
-    if safe_is_subclass(type_, Generic):
+        if isinstance(type_, _compat.TYPE_ALIAS_TYPES):
+            return type_.__parameters__
+
+    if safe_is_subclass(type_, Generic):  # type: ignore (wtf?)
         # Classes that inherit from Generic directly (like
-        # ``class Protocol(Generic):``) and Generic itself don't
+        # `class Protocol(Generic):`) and Generic itself don't
         # have __orig_bases__, while classes that have type
         # parameters do.
         if not hasattr(type_, "__orig_bases__"):
             return None
 
-        return type_.__parameters__  # type: ignore
+        return type_.__parameters__
 
     if isinstance(type_, PARAMETERIZED_GENERIC_META):  # type: ignore
         return type_.__parameters__
@@ -388,6 +390,12 @@ def _is_typing_type(cls):
     if isinstance(cls, GenericAliases):
         return True
 
+    if is_forwardref(cls, raising=False):
+        return False
+
+    if isinstance(cls, TypeVar):
+        return True
+
     if sys.version_info >= (3, 10):
         if cls is types.UnionType or isinstance(cls, types.UnionType):
             return True
@@ -421,7 +429,7 @@ def _get_name(cls):
 if sys.version_info >= (3, 9):
 
     def _is_generic_base_class(cls):
-        if safe_is_subclass(cls, Generic):
+        if safe_is_subclass(cls, Generic):  # type: ignore[wtf]
             return bool(cls.__parameters__)  # type: ignore
 
         return False
@@ -503,22 +511,46 @@ def _get_forward_ref_code(ref):
 
 
 def _is_regular_type(type_):
+    if _is_forwardref(type_):
+        return True
+
     if isinstance(type_, type):
         return True
 
     if type_ is None:
         return True
 
+    # It's a bit weird for this to be here because it's hardly "regular", but all that really
+    # matters is that it's not from `typing`. (Because then it would belong in `is_typing_type()`.)
+    if type_ is _compat.DATACLASSES_KW_ONLY:
+        return True
+
+    if isinstance(type_, GenericAliases):
+        return True
+
+    if type_ in GENERICS_THAT_DONT_INHERIT_FROM_GENERIC:
+        return True
+
+    if isinstance(type_, GENERICS_THAT_DONT_INHERIT_FROM_GENERIC):
+        return True
+
+    if isinstance(type_, _compat.TYPE_ALIAS_TYPES):
+        return True
+
     return False
 
 
+def _is_forwardref(type_):
+    return isinstance(type_, (str, ForwardRef))
+
+
 GENERICS_THAT_DONT_INHERIT_FROM_GENERIC = cast(
     Tuple[type, ...],
     _resolve_dotted_names(("dataclasses.InitVar",)),
 )
 
 
-def is_forwardref(type_: Type_, raising: bool = True) -> bool:
+def is_forwardref(type_: Type_, raising: bool = True) -> typing_extensions.TypeGuard[ForwardRef]:
     """
     Returns whether ``type_`` is a forward reference.
 
@@ -537,7 +569,7 @@ def is_forwardref(type_: Type_, raising: bool = True) -> bool:
     :param raising: Whether to throw a :exc:`NotAType` exception if ``type_`` is not a type
     :return: Whether the object is a class or type (or forward reference)
     """
-    if isinstance(type_, (str, ForwardRef)):
+    if _is_forwardref(type_):
         return True
 
     if raising and not is_type(type_):
@@ -572,23 +604,10 @@ def is_type(type_: Any, allow_forwardref: bool = True) -> bool:
     :param allow_forwardref: Controls whether strings and ForwardRefs are considered types
     :return: Whether the object is a class or type (or forward reference)
     """
-    # strings are forward references
-    if isinstance(type_, (str, ForwardRef)):
+    if _is_forwardref(type_):
         return allow_forwardref
 
-    if isinstance(type_, GenericAliases):
-        return True
-
-    if _is_regular_type(type_):
-        return True
-
-    if type_ in GENERICS_THAT_DONT_INHERIT_FROM_GENERIC:
-        return True
-
-    if isinstance(type_, GENERICS_THAT_DONT_INHERIT_FROM_GENERIC):
-        return True
-
-    return is_typing_type(type_, raising=False)
+    return _is_typing_type(type_) or _is_regular_type(type_)
 
 
 def is_typing_type(type_: Type_, raising: bool = True) -> bool:
@@ -607,15 +626,9 @@ def is_typing_type(type_: Type_, raising: bool = True) -> bool:
     :return: Whether the object is a ``typing`` type
     :raises NotAType: If ``type_`` is not a type and ``raising`` is ``True``
     """
-    if is_forwardref(type_, raising=False):
-        return False
-
     if _is_typing_type(type_):
         return True
 
-    if isinstance(type_, TypeVar):
-        return True
-
     if not raising or _is_regular_type(type_):
         return False
 
@@ -818,7 +831,7 @@ def get_generic_base_class(type_: Type_) -> Type_:
         if len(args) == 1:
             base = Optional
 
-    return base
+    return base  # type: ignore[wtf]
 
 
 def get_type_arguments(type_: Type_) -> Tuple[object, ...]:
@@ -936,7 +949,7 @@ def get_type_parameters(type_: Type_) -> Tuple[TypeParameter, ...]:
     else:
         params = ordered_set.OrderedSet(())
 
-        for base, *typevars in bases:
+        for _, *typevars in bases:
             params.update(typevars)
 
         return tuple(param for param in params if isinstance(param, TypeVar))
@@ -1070,7 +1083,7 @@ def get_type_argument_for(
             arg = args[i]
         except IndexError:
             if assume_any:
-                return Any
+                return Any  # type: ignore
 
             raise TypeVarNotSet(type_var, base_type, type_)  # type: ignore
 
@@ -1150,11 +1163,11 @@ def get_parent_types(type_: Type_) -> Tuple[Type_, ...]:
     try:
         orig_bases = type_.__orig_bases__  # type: ignore
     except AttributeError:
-        return type_.__bases__
+        return type_.__bases__  # type: ignore[wtf]
 
     parent_types = []
 
-    for base, orig_base in zip(type_.__bases__, orig_bases):
+    for base, orig_base in zip(type_.__bases__, orig_bases):  # type: ignore[wtf]
         # Non-generic types show up as-is in both tuples
         if base is orig_base:
             parent_types.append(base)
