@@ -3,6 +3,7 @@ from __future__ import annotations
 import ast
 import builtins
 import collections.abc
+import dataclasses
 import importlib
 import types
 import typing
@@ -22,33 +23,30 @@ from .introspection import (
 from .i_hate_circular_imports import parameterize
 from ..parameter import Parameter
 from ..signature_ import Signature
-from ..types import Type_, TypeAnnotation, ForwardReference, ForwardRefContext
+from ..types import Type_, TypeAnnotation, ForwardRefContext
 from ..errors import *
 
 __all__ = [
-    "is_forward_ref",
     "resolve_forward_refs",
     "annotation_to_string",
     "annotation_for_callable",
 ]
 
 
-def is_forward_ref(annotation: TypeAnnotation) -> typing_extensions.TypeGuard[ForwardReference]:
-    """ """
-    return isinstance(annotation, (str, ForwardRef))
+KW_ONLY = getattr(dataclasses, "KW_ONLY", object())
 
 
 @overload
 def resolve_forward_refs(
     annotation: TypeAnnotation,
-    context: ForwardRefContext = None,
+    context: Optional[ForwardRefContext] = None,
     *,
     mode: Literal["eval", "getattr", "ast"] = "eval",
     strict: bool = True,
     max_depth: Optional[int] = None,
+    extra_globals: Mapping[str, object] = {},
     treat_name_errors_as_imports: bool = False,
-) -> TypeAnnotation:
-    ...
+) -> TypeAnnotation: ...
 
 
 @overload
@@ -57,20 +55,21 @@ def resolve_forward_refs(
     module: typing.Optional[types.ModuleType] = None,
     eval_: bool = True,
     strict: bool = True,
-) -> TypeAnnotation:
-    ...
+) -> TypeAnnotation: ...
 
 
-def resolve_forward_refs(  # type: ignore[wtf]
+def resolve_forward_refs(  # type: ignore
     annotation: TypeAnnotation,
-    context: ForwardRefContext = None,
+    context: Optional[ForwardRefContext] = None,
     eval_: Optional[bool] = None,
     strict: bool = True,
     *,
     module: typing.Optional[types.ModuleType] = None,
     mode: Literal["eval", "getattr", "ast"] = "eval",
     max_depth: Optional[int] = None,
+    extra_globals: Mapping[str, object] = {},
     treat_name_errors_as_imports: bool = False,
+    _currently_evaluating: AbstractSet[Tuple[str, int]] = set(),
 ) -> TypeAnnotation:
     """
     Resolves forward references in a type annotation.
@@ -128,6 +127,8 @@ def resolve_forward_refs(  # type: ignore[wtf]
             mode=mode,
             strict=strict,
             max_depth=max_depth - 1,
+            extra_globals=extra_globals,
+            _currently_evaluating=_currently_evaluating,  # type: ignore
         )
 
     if isinstance(annotation, ForwardRef):
@@ -137,11 +138,19 @@ def resolve_forward_refs(  # type: ignore[wtf]
         annotation = _get_forward_ref_code(annotation)
 
     if isinstance(annotation, str):
+        # First, check if this exact same forward reference is already being evaluated - i.e. it is
+        # a recursive type.
+        key = (annotation, id(context))
+        if key in _currently_evaluating:
+            return annotation
+        _currently_evaluating = _currently_evaluating | {key}
+
         scope: collections.ChainMap[str, object] = collections.ChainMap()
 
         if context is None:
             scope.maps.extend(
-                vars(module) for module in (collections.abc, collections, typing, typing_extensions)  # type: ignore
+                vars(module)
+                for module in (collections.abc, collections, typing, typing_extensions)  # type: ignore
             )
         elif isinstance(context, types.ModuleType):
             scope.maps.append(vars(context))
@@ -155,12 +164,15 @@ def resolve_forward_refs(  # type: ignore[wtf]
             scope.maps.append(vars(module))
 
         scope.maps.append(vars(builtins))  # type: ignore
+        scope.maps.append(extra_globals)  # type: ignore
 
         if treat_name_errors_as_imports:
             from ._utils import ImporterDict
 
             scope.maps.append(ImporterDict())  # type: ignore
 
+        # Note: Annotations can be strings inside of strings, like `"'int'"`. So evaluating it once
+        # isn't necessarily enough; make sure to call `recurse()` with the result!
         if mode == "eval":
             try:
                 # The globals must be a real dict, so the scope will be used as
@@ -182,17 +194,23 @@ def resolve_forward_refs(  # type: ignore[wtf]
                     for attr in attrs:
                         value = getattr(value, attr)
 
-                    return value  # type: ignore
+                    return recurse(value)  # type: ignore
                 except AttributeError:
                     pass
         elif mode == "ast":
             expr = ast.parse(annotation, mode="eval")
             try:
-                result = _eval_ast(expr.body, scope, strict=strict, max_depth=max_depth)
+                result = _eval_ast(
+                    expr.body,
+                    scope,
+                    strict=strict,
+                    max_depth=max_depth,
+                    treat_name_errors_as_imports=treat_name_errors_as_imports,
+                )
             except Exception:
                 pass
             else:
-                return result  # type: ignore
+                return recurse(result)  # type: ignore
         else:
             assert False, f"Invalid mode: {mode!r}"
 
@@ -263,20 +281,22 @@ def resolve_forward_refs(  # type: ignore[wtf]
 
 
 def _eval_ast(
-    node: ast.AST, scope: typing.Mapping[str, object], strict: bool, max_depth: int
+    node: ast.AST,
+    scope: typing.Mapping[str, object],
+    strict: bool,
+    max_depth: int,
+    treat_name_errors_as_imports: bool,
 ) -> object:
     # Compared to "eval" and "getattr", this method of evaluating forward refs
     # has the advantage of being able to perform partial evaluation. For
     # example, the forward ref `"ClassVar[NameThatCannotBeResolved]"` can be
     # turned into `ClassVar["NameThatCannotBeResolved"]`.
-    #
-    # Sometimes we need to know whether the forward ref was resolved or not.
-    # That's why this function returns a tuple of `(bool, object)`.
+
     def recurse(node: ast.AST) -> object:
         if max_depth <= 1:
             return ast.unparse(node)
 
-        return _eval_ast(node, scope, strict, max_depth - 1)
+        return _eval_ast(node, scope, strict, max_depth - 1, treat_name_errors_as_imports)
 
     if strict:
         safe_recurse = recurse
@@ -298,7 +318,7 @@ def _eval_ast(
             mode="ast",
             strict=strict,
             max_depth=max_depth - 1,
-            treat_name_errors_as_imports=False,
+            treat_name_errors_as_imports=treat_name_errors_as_imports,
         )
 
     if type(node) is ast.Name:
@@ -306,7 +326,26 @@ def _eval_ast(
         return scope[name]
     elif type(node) is ast.Attribute:
         obj = recurse(node.value)
-        return getattr(obj, node.attr)
+
+        try:
+            return getattr(obj, node.attr)
+        except AttributeError:
+            # The parameter name is a little misleading, but we'll also treat AttributeErrors on
+            # modules as missing imports
+            if not treat_name_errors_as_imports or not isinstance(obj, types.ModuleType):
+                raise
+
+            try:
+                importlib.import_module(f"{obj.__name__}.{node.attr}")
+            except ImportError:
+                pass
+
+            try:
+                return getattr(obj, node.attr)
+            except AttributeError:
+                pass
+
+            raise
     elif type(node) is ast.Subscript:
         generic_type = recurse(node.value)
         subtype = safe_recurse(node.slice)
@@ -349,9 +388,11 @@ def annotation_to_string(
     annotation: TypeAnnotation,
     *,
     implicit_typing: bool = True,
+    implicit_dataclasses: bool = True,
     new_style_unions: bool = True,
     optional_as_union: bool = True,
     variance_prefixes: bool = False,
+    aliases: Mapping[object, str] = {},
 ) -> str:
     """
     Converts a type annotation to string. The result is valid python code
@@ -369,6 +410,7 @@ def annotation_to_string(
     :param annotation: A class or type annotation
     :param implicit_typing: Whether to omit the "typing." prefix from ``typing``
         types' names
+    :param implicit_dataclasses: Whether to omit the "dataclasses." prefix from ``dataclasses.KW_ONLY``
     :param new_style_unions: Whether to use the new-style ``typing.Union`` syntax
         ``int | str`` instead of ``Union[int, str]``
     :param variance_prefixes: Whether `TypeVars` and `ParamSpecs` should be
@@ -383,6 +425,7 @@ def annotation_to_string(
             new_style_unions=new_style_unions,
             optional_as_union=optional_as_union,
             variance_prefixes=variance_prefixes,
+            aliases=aliases,
         )
 
     def process_nested(prefix: str, elems: Iterable[TypeAnnotation]):
@@ -395,9 +438,20 @@ def annotation_to_string(
     if isinstance(annotation, str):
         return annotation
 
+    try:
+        return aliases[annotation]
+    except (KeyError, TypeError):
+        pass
+
     if annotation is ...:
         return "..."
 
+    if annotation is KW_ONLY:
+        if implicit_dataclasses:
+            return "KW_ONLY"
+        else:
+            return "dataclasses.KW_ONLY"
+
     if annotation in (None, type(None)):
         return "None"
 
@@ -415,7 +469,7 @@ def annotation_to_string(
         if base in (typing.Callable, collections.abc.Callable):
             param_types, return_type = subtypes
 
-            prefix = recurse(base)
+            prefix = recurse(base)  # type: ignore[wtf]
             return_str = recurse(return_type)  # type: ignore
 
             if isinstance(param_types, list):
@@ -428,7 +482,7 @@ def annotation_to_string(
 
         if base in LITERAL_TYPES:
             literals = ", ".join(repr(value) for value in subtypes)
-            prefix = recurse(base)
+            prefix = recurse(base)  # type: ignore[wtf]
             return f"{prefix}[{literals}]"
 
         if base is typing_extensions.Annotated:
@@ -436,10 +490,10 @@ def annotation_to_string(
             sub_strs = [recurse(sub_type)]  # type: ignore
             sub_strs.extend(repr(ann) for ann in annotations)
 
-            prefix = recurse(base)
-            return f'{prefix}[{", ".join(sub_strs)}]'
+            prefix = recurse(base)  # type: ignore[wtf]
+            return f"{prefix}[{', '.join(sub_strs)}]"
 
-        prefix = recurse(base)
+        prefix = recurse(base)  # type: ignore[wtf]
         return process_nested(prefix, subtypes)  # type: ignore
 
     if isinstance(annotation, (typing.TypeVar, typing_extensions.ParamSpec)):
@@ -495,7 +549,7 @@ def annotation_for_callable(callable_: typing.Callable[..., object]) -> Type_:
     .. versionadded:: 1.5
     """
     signature = Signature.from_callable(callable_)
-    parameters = signature.parameters.values()
+    parameters = list(signature.parameters.values())
 
     if signature.return_annotation is Signature.empty:
         return_type = typing.Any
