@@ -1,38 +1,22 @@
 import pytest
 
+import abc
+import builtins
+import dataclasses
 import functools
 import inspect
-import sys
-import typing
+import typing as t
 
 import introspection
 from introspection import Signature, Parameter, errors
 
 
-def make_fake_c_function(doc, monkeypatch):
-    # Functions written in C don't have signatures; we'll fake one by creating
-    # an object with a __doc__ attribute
-    def fake_sig(*a, **kw):
-        raise ValueError("no signature found")
-
-    monkeypatch.setattr(inspect, "signature", fake_sig)
-    monkeypatch.setattr(
-        sys.modules["introspection.signature_"], "callable", lambda _: True, raising=False
-    )
-
-    class FakeFunction:
-        pass
-
-    fake_func = FakeFunction()
-    fake_func.__doc__ = doc
-    return fake_func
-
-
 def test_get_signature():
     def foo(a, b=3) -> str:
         return ""
 
     sig = Signature.from_callable(foo)
+    assert sig.forward_ref_context == __name__
     assert sig.return_annotation is str
     assert len(sig.parameters) == 2
     assert list(sig.parameters) == ["a", "b"]
@@ -44,6 +28,7 @@ def test_get_signature():
 
 def test_get_int_signature():
     sig = Signature.from_callable(int)
+    assert sig.forward_ref_context in (None, "builtins")
     assert sig.return_annotation is int
     assert len(sig.parameters) == 2
     assert list(sig.parameters) == ["x", "base"]
@@ -55,6 +40,7 @@ def test_get_int_signature():
 
 def test_get_float_signature():
     sig = Signature.from_callable(float)
+    assert sig.forward_ref_context in (None, "builtins")
     assert sig.return_annotation is float
     assert len(sig.parameters) == 1
     assert list(sig.parameters) == ["x"]
@@ -64,6 +50,7 @@ def test_get_float_signature():
 
 def test_get_bool_signature():
     sig = Signature.from_callable(bool)
+    assert sig.forward_ref_context in (None, "builtins")
     assert sig.return_annotation is bool
     assert len(sig.parameters) == 1
     assert list(sig.parameters) == ["x"]
@@ -71,15 +58,89 @@ def test_get_bool_signature():
     assert sig.parameters["x"].default is Parameter.missing
 
 
-def test_get_signature_undoc_c_function(monkeypatch):
-    fake_func = make_fake_c_function(None, monkeypatch)
+def test_get_signature_undoc_c_function():
+    c_function = iter
+
+    try:
+        inspect.signature(c_function)
+    except ValueError:
+        pass
+    else:
+        pytest.fail(
+            f"{c_function} is no longer suitable for this test."
+            f" We need `inspect.signature` to throw a `ValueError`"
+        )
 
     with pytest.raises(errors.NoSignatureFound):
-        Signature.from_callable(fake_func)  # type: ignore
+        Signature.from_callable(c_function, use_signature_db=False)
 
     # Deprecated exception
     with pytest.raises(ValueError):
-        Signature.from_callable(fake_func)  # type: ignore
+        Signature.from_callable(c_function, use_signature_db=False)
+
+
+def test_get_dataclass_signature():
+    @dataclasses.dataclass
+    class MyDataClass:
+        foo: int
+
+    sig = Signature.from_callable(MyDataClass)
+    assert list(sig.parameters) == ["foo"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_get_abstract_class_signature():
+    class MyAbstractClass(abc.ABC):
+        def __init__(self, foo: str):
+            pass
+
+        @abc.abstractmethod
+        def func(self):
+            pass
+
+    sig = Signature.from_callable(MyAbstractClass)
+    assert list(sig.parameters) == ["foo"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_constructor_descriptors():
+    call_args = []
+    new_args = []
+    init_args = []
+
+    class CallDescriptor:
+        def __get__(self, *args):
+            call_args.append(args)
+            return type.__call__.__get__(*args)
+
+    class NewDescriptor:
+        def __get__(self, *args):
+            new_args.append(args)
+            return object.__new__  # No `.__get__` here, that crashes for some reason...
+
+    class InitDescriptor:
+        # `Signature.from_callable` calls `__init__` with a dummy instance, so it makes no sense to
+        # record it.
+        def __get__(self, instance, *args):
+            init_args.append(args)
+            return object.__init__.__get__(instance, *args)
+
+    class Meta(type):
+        __call__ = introspection.mark.does_not_alter_signature(CallDescriptor())  # type: ignore
+
+    class Class(metaclass=Meta):
+        __new__ = introspection.mark.does_not_alter_signature(NewDescriptor())  # type: ignore
+        __init__ = introspection.mark.does_not_alter_signature(InitDescriptor())  # type: ignore
+
+    # First, see how python itself invokes the descriptors
+    Class()
+
+    # Then obtain the class's signature and compare
+    Signature.from_callable(Class)
+
+    assert call_args[0] == call_args[1]
+    assert new_args[0] == new_args[1]
+    assert init_args[0] == init_args[1]
 
 
 def test_get_signature_noncallable():
@@ -92,7 +153,7 @@ def test_signature_with_optional_parameter():
 
     assert sig.return_annotation is dict
     assert len(sig.parameters) == 1
-    assert sig.parameters["object"].annotation is typing.Any
+    assert sig.parameters["object"].annotation is t.Any
     assert sig.parameters["object"].default is Parameter.missing
 
 
@@ -101,24 +162,62 @@ def test_store_signature():
         return "bar"
 
     sig = Signature.from_callable(foo)
-    foo.__signature__ = sig
+    foo.__signature__ = sig  # type: ignore
 
     s = inspect.signature(foo)
     assert s is sig
 
 
-def test_builtin_signatures():
-    import builtins
+def test_cached_signature():
+    def foo(a=5) -> str:
+        return "bar"
 
-    for thing in vars(builtins).values():
-        if not callable(thing):
-            continue
+    # Create and cache an `inspect.Signature`
+    foo.__signature__ = inspect.signature(foo)  # type: ignore
 
-        try:
-            _ = Signature.from_callable(thing)
-        except Exception as e:
-            msg = "Couldn't obtain signature of {!r}: {!r}"
-            pytest.fail(msg.format(thing, e))
+    # Now, when we create a `Signature` object, make sure that metadata such as
+    # `forward_ref_context` is still available
+    sig = Signature.from_callable(foo)
+    assert sig.forward_ref_context == __name__
+    assert sig.parameters["a"].forward_ref_context == __name__
+
+
+def test_class_signature_cache():
+    # Make sure classes don't inherit the cached signature of their parent
+    class Parent:
+        def __init__(self, parent_param):
+            pass
+
+    class Child(Parent):
+        def __init__(self, child_param):
+            pass
+
+    _ = Signature.from_callable(Parent)
+    sig = Signature.from_callable(Child)
+
+    assert list(sig.parameters) == ["child_param"]
+
+
+BUILTIN_CALLABLES = {
+    name: obj
+    for name, obj in vars(builtins).items()
+    if callable(obj) and obj.__module__ == "builtins"
+}
+
+
+@pytest.mark.parametrize(
+    "callable_",
+    BUILTIN_CALLABLES.values(),
+    ids=BUILTIN_CALLABLES.keys(),
+)
+def test_builtin_signatures(callable_: t.Callable):
+    try:
+        _ = Signature.from_callable(callable_)
+    except Exception as e:
+        pytest.fail(
+            f"Couldn't obtain signature of {callable_.__name__!r}: {e!r}."
+            f" It must be added to the signature database."
+        )
 
 
 def test_follow_wrapped():
@@ -146,18 +245,31 @@ def test_dont_follow_wrapped():
     assert list(sig.parameters) == ["args", "kwargs"]
 
 
-def test_bound_method_signature():
+def test_unbound_method_signature():
     class A:
         def method(self, foo: int) -> str:
             return "hi"
 
+    sig = introspection.signature(A.method)
+
+    assert list(sig.parameters) == ["self", "foo"]
+    assert sig.return_annotation is str
+    assert sig.parameters["foo"].annotation is int
+    assert sig.parameters["foo"].kind is Parameter.POSITIONAL_OR_KEYWORD
+
+
+def test_bound_method_signature():
+    class A:
+        def method(self, foo: int, /) -> str:
+            return "hi"
+
     obj = A()
     sig = introspection.signature(obj.method)
 
     assert list(sig.parameters) == ["foo"]
     assert sig.return_annotation is str
     assert sig.parameters["foo"].annotation is int
-    assert sig.parameters["foo"].kind is Parameter.POSITIONAL_OR_KEYWORD
+    assert sig.parameters["foo"].kind is Parameter.POSITIONAL_ONLY
 
 
 def test_decorated_bound_method_signature():
@@ -219,6 +331,149 @@ def test_method_signature_with_repeated_argument():
     assert sig.parameters["foo"].kind is Parameter.KEYWORD_ONLY
 
 
+def test_class_signature():
+    class Cls:
+        def __init__(self, init):
+            pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == ["init"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_class_signature_with_no_constructor():
+    class Cls:
+        pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == []
+
+
+def test_class_signature_with_metaclass():
+    class Meta(type):
+        def __call__(self, meta):  # type: ignore (invalid override)
+            pass
+
+    class Cls(metaclass=Meta):
+        def __new__(cls, new):
+            pass
+
+        def __init__(self, init):
+            pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == ["meta"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_skip_metaclass_signature():
+    class Meta(type):
+        @introspection.mark.does_not_alter_signature
+        def __call__(cls, *args, **kwargs):
+            return super().__call__(*args, **kwargs)
+
+    class Cls(metaclass=Meta):
+        def __init__(self, foo):
+            pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == ["foo"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_mark_on_decorated_function():
+    def wrap_in_unnecessary_decorator(func):
+        @functools.wraps(func)
+        def wrapper(*args, **kwargs):
+            return func(*args, **kwargs)
+
+        return wrapper
+
+    class Meta(type):
+        # Make sure the mark still takes effect if the function is decorated
+        @wrap_in_unnecessary_decorator
+        @introspection.mark.does_not_alter_signature
+        def __call__(cls, *args, **kwargs):
+            return super().__call__(*args, **kwargs)
+
+    class Cls(metaclass=Meta):
+        def __init__(self, foo):
+            pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == ["foo"]
+    assert sig.forward_ref_context == __name__
+
+
+def test_partial():
+    def func(x, *y, z):
+        pass
+
+    part = functools.partial(func, 1, 2, 3, z=4)
+    sig = Signature.from_callable(part)
+    assert list(sig.parameters) == ["y", "z"]
+    assert sig.parameters["y"].kind is Parameter.VAR_POSITIONAL
+    assert sig.parameters["z"].default == 4
+
+
+def test_partial_and_skip_metaclass_signature():
+    class Meta(type):
+        @introspection.mark.does_not_alter_signature
+        def __call__(cls, *args, **kwargs):
+            return super().__call__(*args, **kwargs)
+
+    class Cls(metaclass=Meta):
+        def __init__(self, foo, bar: "int"):
+            pass
+
+    sig = Signature.from_callable(functools.partial(Cls, 3))
+    assert list(sig.parameters) == ["bar"]
+
+
+def test_builtin_class_signature():
+    # Just make sure it doesn't crash
+    _ = Signature.from_callable(float, use_signature_db=False)
+
+
+def test_custom_callable():
+    class CustomCallable:
+        def __call__(self, param: str):
+            pass
+
+    sig = Signature.from_callable(CustomCallable())
+    assert list(sig.parameters) == ["param"]
+    assert sig.parameters["param"].annotation is str
+
+
+def test_custom_callable_with_different_modules():
+    from collections import UserString
+
+    class CustomCallable:
+        def __call__(self, param: "UserString"):
+            pass
+
+    # Pretend the `__call__` was inherited from a base class that was defined somewhere else
+    CustomCallable.__module__ = "sys"
+    CustomCallable.__call__.__module__ = "collections"
+
+    sig = Signature.from_callable(CustomCallable())
+    assert list(sig.parameters) == ["param"]
+    assert sig.forward_ref_context == "collections"
+
+
+def test_doesnt_alter_signature_mark():
+    class Cls:
+        @introspection.mark.does_not_alter_signature
+        def __new__(cls, *args, **kwargs):
+            return super().__new__(cls, *args, **kwargs)
+
+        def __init__(self, init):
+            pass
+
+    sig = Signature.from_callable(Cls)
+    assert list(sig.parameters) == ["init"]
+
+
 def test_replace():
     sig = Signature([Parameter("foo")], return_annotation=int)
 
@@ -230,14 +485,41 @@ def test_replace():
 
 
 def test_without_parameters():
-    sig = Signature([Parameter("foo"), Parameter("bar"), Parameter("baz")])
+    sig = Signature(
+        [
+            Parameter("foo"),
+            Parameter("bar"),
+            Parameter("baz", Parameter.KEYWORD_ONLY),
+            Parameter("qux", Parameter.VAR_KEYWORD),
+        ]
+    )
 
-    expected = Signature([Parameter("bar")])
+    expected = Signature([Parameter("qux", Parameter.VAR_KEYWORD)])
 
-    result = sig.without_parameters(0, "baz")
+    result = sig.without_parameters(0, "bar", Parameter.KEYWORD_ONLY)
     assert result == expected
 
 
+def test_without_parameters_by_kind():
+    sig = Signature(
+        [
+            Parameter("foo", Parameter.VAR_POSITIONAL),
+            Parameter("bar", Parameter.KEYWORD_ONLY),
+            Parameter("baz", Parameter.VAR_KEYWORD),
+        ]
+    )
+
+    # For some inexplicable reason, `ParameterKind` is an `IntEnum`. Make sure it doesn't get
+    # confused with indices.
+    assert Parameter.POSITIONAL_ONLY == 0
+    assert Parameter.VAR_KEYWORD == 4
+    result = sig.without_parameters(Parameter.POSITIONAL_ONLY, 4)
+
+    # Since there is no positional-only parameter and also no parameter with index 4, the signature
+    # should remain unchanged.
+    assert result == sig
+
+
 def test_num_required_arguments():
     sig = Signature(
         [
@@ -403,11 +685,11 @@ def test_bind_partial(func, args, kwargs, expected_result):
         (Signature([Parameter("x", annotation=int)], return_annotation=str), "(x: int) -> str"),
         (Signature([Parameter("x", annotation=bool, default=False)]), "(x: bool = False)"),
         (
-            Signature([Parameter("x", annotation=tuple)], return_annotation=typing.Tuple),
+            Signature([Parameter("x", annotation=tuple)], return_annotation=t.Tuple),
             "(x: tuple) -> typing.Tuple",
         ),
         (
-            Signature(return_annotation=typing.Tuple[int, typing.List]),
+            Signature(return_annotation=t.Tuple[int, t.List]),
             "() -> typing.Tuple[int, typing.List]",
         ),
     ],
