@@ -1,10 +1,12 @@
 import pytest
 
 import collections.abc
+import dataclasses
 import re
 import sys
 import types
 import typing
+import typing as t
 import typing_extensions
 from typing import *
 
@@ -44,7 +46,7 @@ class UnhashableClass(metaclass=UnhashableMeta):
         (UnhashableClass, False),
         (List, False),
         ("Foo", True),
-        (ForwardRef("Foo"), True),
+        (t.ForwardRef("Foo"), True),
     ],
 )
 def test_is_forwardref(type_, expected):
@@ -89,6 +91,8 @@ def test_is_forwardref_non_raising(obj):
         (3, False),
         ([], False),
         ("Foo", True),  # this is a forward reference
+        ("List", True),
+        (t.ForwardRef("List"), True),
         (T_co, True),
         (E, True),
         (Any, True),
@@ -124,7 +128,7 @@ def test_is_type(type_, expected):
     "type_, expected",
     [
         ("Foo", False),
-        (ForwardRef("Foo"), False),
+        (t.ForwardRef("Foo"), False),
     ],
 )
 def test_is_type_no_forwardref(type_, expected):
@@ -139,7 +143,7 @@ def test_is_type_no_forwardref(type_, expected):
         (UnhashableClass, False),
         (None, False),
         ("List", False),
-        (ForwardRef("List"), False),
+        (t.ForwardRef("List"), False),
         (T_co, True),
         (E, True),
         (Any, True),
@@ -1009,11 +1013,11 @@ if sys.version_info >= (3, 10):
 
     def test_uniontype_get_type_arguments_error():
         with pytest.raises(errors.NotAParameterizedGeneric):
-            get_type_arguments(types.UnionType)
+            get_type_arguments(types.UnionType)  # type: ignore
 
         # Deprecated exception
         with pytest.raises(ValueError):
-            get_type_arguments(types.UnionType)
+            get_type_arguments(types.UnionType)  # type: ignore
 
     @pytest.mark.parametrize(
         "type_, expected",
@@ -1030,8 +1034,28 @@ if sys.version_info >= (3, 10):
 
     def test_uniontype_get_type_parameters_error():
         with pytest.raises(errors.NotAGeneric):
-            get_type_parameters(types.UnionType)
+            get_type_parameters(types.UnionType)  # type: ignore
 
         # Deprecated exception
         with pytest.raises(ValueError):
-            get_type_parameters(types.UnionType)
+            get_type_parameters(types.UnionType)  # type: ignore
+
+
+if hasattr(dataclasses, "KW_ONLY"):
+
+    def test_kw_only():
+        assert is_type(dataclasses.KW_ONLY)
+
+
+if sys.version_info >= (3, 12):
+    NewStyleTypeAlias = tuple  # Shut up the type checker
+    # Use exec to prevent a syntax error in older versions
+    exec("type NewStyleTypeAlias[A, B] = list[A] | tuple[B]")
+
+    def test_new_style_type_alias():
+        assert is_type(NewStyleTypeAlias)
+        assert not is_typing_type(NewStyleTypeAlias)
+        assert is_generic(NewStyleTypeAlias)
+        assert not is_generic_base_class(NewStyleTypeAlias)
+        assert not is_parameterized_generic(NewStyleTypeAlias)
+        assert is_parameterized_generic(NewStyleTypeAlias[int, str])
